from os import getenv
from slack_sdk import WebClient
from slack_sdk.errors import SlackApiError
from .logger import get_logger


def post_to_slack(message="", channel="#security-alerts", token=None):
    """post message to slack

    :param message: message to send
    :type message: str
    :param channel: Slack channel name(default=#security-alerts)
    :type channel: str
    :param token: bot or user access token. If token is none, SLACK_API_TOKEN environment value will be assigned.
    :type token: str
    :return: `True` if successful, `None` otherwise
    """
    logger = get_logger(__name__)
    token = getenv("SLACK_API_TOKEN", token)
    if not SLACK_API_TOKEN:
        logger.error("SLACK_API_TOKEN not found in the environment")
        return None
    client = WebClient(token=token)
    try:
        response = client.chat_postMessage(channel=channel, text=message)
    except SlackApiError as e:
        logger.error(f"Got an error: {e.response['error']}")
        return None
    return True



from os import getenv
import hvac
from .logger import get_logger


def _get_client(
    vault_url=None, vault_username=None, vault_password=None, vault_skip_verify=True
):
    logger = get_logger(__name__)
    vault_url = getenv("VAULT_URL", vault_url)
    vault_username = getenv("VAULT_USERNAME", vault_username)
    vault_password = getenv("VAULT_PASSWORD", vault_password)
    vault_skip_verify = getenv("VAULT_SKIP_VERIFY", vault_skip_verify)
    if not vault_url:
        logger.error(
            "vault_url argument is not provided and VAULT_URL is not present in the environment"
        )
        return None
    if not vault_username:
        logger.error(
            "vault_username argument is not provided and VAULT_USERNAME is not present in the environment"
        )
        return None
    if not vault_password:
        logger.error(
            "vault_password argument is not provided and VAULT_PASSWORD is not present in the environment"
        )
        return None
    client = hvac.Client(url=vault_url, verify=vault_skip_verify)
    login_response = client.auth_userpass(
        username=vault_username,
        password=vault_password,
    )
    if client.is_authenticated():
        return client
    return None


def get_vault_secrets(
    path="",
    vault_url=None,
    vault_username=None,
    vault_password=None,
    vault_skip_verify=True,
):
    """get vault secrets for the path

    :param path: vault secrets path
    :type path: str
    :param vault_url: vault server url. If vault_url is none, VAULT_URL environment value will be assigned.
    :type vault_url: str
    :param vault_username: vault username. If vault_username is none, VAULT_USERNAME environment value will be assigned.
    :type vault_username: str
    :param vault_password: vault password. If vault_password is none, VAULT_PASSWORD environment value will be assigned.
    :type vault_password: str
    :param vault_skip_verify: vault skip verification. If VAULT_SKIP_VERIFY is none, VAULT_PASSWORD environment value will be assigned.
    :type vault_skip_verify: bool
    :return: List(Dict[str, Any]) if successful, `None` otherwise
    """
    logger = get_logger(__name__)
    client = _get_client(
        vault_url=vault_url,
        vault_username=vault_username,
        vault_password=vault_password,
        vault_skip_verify=vault_skip_verify,
    )
    if client is None:
        return None
    try:
        secrets = client.read(path)
        client.revoke_self_token()
        if "data" in secrets:
            return secrets["data"]
    except Exception as e:
        logger.error(f"Got an error: {e.message}")
        return None
    return None



import json
from os import getenv
import yaml
from .vault import get_vault_secrets
from .logger import get_logger
import re

_from_env = True
_vault_secrets = None
_json_configs = None
_yaml_configs = None


def init_config(from_env=True, json_file=None, yaml_file=None, vault_path=None):
    """initializes the config module

    :param from_env: should the config read from environment.
    :type from_env: bool
    :param json_file: json config file path.
    :type json_file: str
    :param yaml_file: yaml config file path.
    :type yaml_file: str
    :param vault_path: should the config read from valut server. If vault_path is set, The VAULT_URL & VAULT_TOKEN environments should be set.
    :type vault_path: str
    """

    global _from_env, _vault_secrets, _json_configs, _yaml_configs
    logger = get_logger(__name__)

    _from_env = from_env

    if not (json_file is None):
        try:
            with open(json_file, "r") as f:
                _json_configs = json.load(f)
        except Exception as e:
            logger.error(f"Got an error: {e.message}")

    if not (yaml_file is None):
        try:
            with open(yaml_file, "r") as f:
                _yaml_configs = yaml.load(f, yaml.SafeLoader)
        except Exception as e:
            logger.error(f"Got an error: {e.message}")
    if not (vault_path is None):
        _vault_secrets = get_vault_secrets(vault_path)


def get_config(key=True, default=None):
    """Get an config variable, return None if it doesn't exist.
    The optional second argument can specify an alternate default.

    :param key: configuration key parameter.
    :type key: str
    :param default: The default value if key doesn't exist.
    """
    global _from_env, _vault_secrets, _json_configs, _yaml_configs

    if _from_env:
        value = getenv(key=key, default=None)
        if not (value is None):
            return value
        else:
            value = getenv(key=re.sub(r"\s+", "_", key).upper(), default=None)
            if not (value is None):
                return value
    if not (_json_configs is None):
        if key in _json_configs:
            return _json_configs[key]
    if not (_yaml_configs is None):
        if key in _yaml_configs:
            return _yaml_configs[key]
    if not (_vault_secrets is None):
        if key in _vault_secrets:
            return _vault_secrets[key]
    return default



import logging


def get_logger(name, level="INFO"):
    """common formated logger

    :param name: name of the logger
    :type name: str
    :param level: level of the logger(INFO,ERROR,DEBUG,WARNING,FATAL)
    :type level: str
    """
    logger = logging.getLogger(name)
    logger.setLevel(getattr(logging, level))
    channel = logging.StreamHandler()
    channel.setLevel(logging.DEBUG)
    formatter = logging.Formatter(
        "%(levelname)-4s %(asctime)-4s [%(filename)s:%(lineno)d] %(message)s",
        datefmt="%Y-%m-%d:%H:%M:%S",
    )
    channel.setFormatter(formatter)
    logger.addHandler(channel)
    return logger
